// FLATBTN.CPP - Implements the CFlatButton class.
// Author - Arnab Bhaduri
///////////////////////////////////////////////////////////////////

#define STRICT

// Includes ///////////////////////////////////////////////////////

#include "stdafx.h"
#include "FlatBtn.h"

// class constants ////////////////////////////////////////////////

int CFlatButton::MenuFraction = 5;
int CFlatButton::BtnMargin    = 3;
int CFlatButton::ArrowSize    = 3;

// message map ////////////////////////////////////////////////////

BEGIN_MESSAGE_MAP(CFlatButton, CButton)
    //{{AFX_MSG_MAP(CFlatButton)
    ON_WM_ERASEBKGND()
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONDOWN()
    ON_WM_LBUTTONUP()
    ON_WM_LBUTTONDBLCLK()
    //}}AFX_MSG_MAP
    ON_WM_CAPTURECHANGED()    // manually added
END_MESSAGE_MAP()


///////////////////////////////////////////////////////////////////
// CFlatButton construction and destruction.
///////////////////////////////////////////////////////////////////


// Constructor ////////////////////////////////////////////////////
CFlatButton::CFlatButton()
{
m_bMouse         = FALSE;
m_bClick         = FALSE;
m_bMenu          = FALSE;
m_pMenu          = NULL;
m_pBitmap        = NULL;
m_nMenuAreaWidth = 0;
}


// Destructor /////////////////////////////////////////////////////
CFlatButton::~CFlatButton()
{
if( m_pMenu )
    delete m_pMenu;
if( m_pBitmap )
    delete m_pBitmap;
}


///////////////////////////////////////////////////////////////////
// CFlatButton members.
///////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////
// Create - ensure owner-draw flag is set for control.
///////////////////////////////////////////////////////////////////
BOOL CFlatButton::Create( LPCTSTR lpszWindowName, DWORD dwStyle,
                          const RECT& rect, CWnd* pParentWnd,
                          UINT nID ) 
{
dwStyle |= BS_OWNERDRAW;

BOOL bRet = CButton::Create( lpszWindowName, dwStyle, rect,
                             pParentWnd, nID );

if( bRet )
    // set menu area proportion for control
    m_nMenuAreaWidth = ( rect.right - rect.left ) / MenuFraction;

return bRet;
}


///////////////////////////////////////////////////////////////////
// OnEraseBkgnd - repaint control surface in standard color.
///////////////////////////////////////////////////////////////////
BOOL CFlatButton::OnEraseBkgnd( CDC* pDC )
{
CRect rect;

GetClientRect( &rect );
pDC->FillSolidRect( &rect, GetSysColor(COLOR_BTNFACE) );
return TRUE;
}


///////////////////////////////////////////////////////////////////
// DrawItem - paint client area according to control state.
///////////////////////////////////////////////////////////////////
void CFlatButton::DrawItem( LPDRAWITEMSTRUCT lpDrawItemStruct )
{
CDC  *pDC = CDC::FromHandle( lpDrawItemStruct->hDC );
CRect rect;

GetBtnAreaRect( &rect );
// leave margin around control's edges
rect.DeflateRect( BtnMargin, BtnMargin );

// set text parameters and draw text
CString text;
GetWindowText( text );

if( text.GetLength() > 0 )
    {
    pDC->SelectStockObject( ANSI_VAR_FONT );
    pDC->SetTextColor( COLOR_BTNTEXT );

    if( m_pBitmap )
        // align text along bottom to leave room for bitmap
        pDC->DrawText( text, &rect,
                       DT_SINGLELINE | DT_CENTER | DT_BOTTOM );
    else
        // no bitmap, align text at center
        pDC->DrawText( text, &rect,
                       DT_SINGLELINE | DT_CENTER | DT_VCENTER );
    }

// draw the bitmap if present
if( m_pBitmap )
    {
    BITMAP bm;
    m_pBitmap->GetBitmap( &bm );

    GetBtnAreaRect( &rect );
    rect.DeflateRect( BtnMargin, BtnMargin );

    // center bitmap horizontally and clip to drawing area
    rect.left  += ( max(0, rect.Width() - bm.bmWidth) ) / 2;
    rect.right -= ( max(0, rect.Width() - bm.bmWidth) ) / 2;

    if( text.GetLength() > 0 )
        // text present, align bitmap to top of drawing area
        rect.bottom = rect.top + bm.bmHeight;
    else
        {
        // no text, center bitmap vertically
        rect.top    += ( max(0, rect.Height() - bm.bmHeight) ) / 2;
        rect.bottom -= ( max(0, rect.Height() - bm.bmHeight) ) / 2;
        }

    CDC dcMem;
    dcMem.CreateCompatibleDC( NULL );

    CBitmap *pOld = dcMem.SelectObject( m_pBitmap );

    pDC->BitBlt( rect.left, rect.top, rect.Width(), rect.Height(),
                 &dcMem, 0, 0, SRCCOPY );
    dcMem.SelectObject( pOld );
    }

// draw the arrow in the menu area if there is a menu - draw it
// slightly offset if the button is "pushed in"
if( m_pMenu )
    {
    POINT pts[3];
    int   nOffset = ( m_bClick | m_bMenu ) ? 1 : 0;

    GetMenuAreaRect( &rect );
    pts[0].x = rect.CenterPoint().x - ArrowSize + nOffset;
    pts[0].y = rect.CenterPoint().y + nOffset;
    pts[1].x = pts[0].x + 2 * ArrowSize;
    pts[1].y = pts[0].y;
    pts[2].x = pts[0].x + ArrowSize;
    pts[2].y = pts[0].y + ArrowSize;

    CRgn rgn;
    rgn.CreatePolygonRgn( pts, 3, WINDING );

    CBrush brush( RGB(0, 0, 0) );
    pDC->FillRgn( &rgn, &brush );
    }

// draw the 3D edges
if( m_bMenu )
    {
    // the menu is being displayed - draw the button area with the
    // "raised" look
    GetBtnAreaRect( &rect );
    pDC->Draw3dRect( &rect,
                     GetSysColor(COLOR_BTNHIGHLIGHT),
                     GetSysColor(COLOR_BTNSHADOW) );
    // draw the menu area with the "pushed-in" look
    GetMenuAreaRect( &rect );
    pDC->Draw3dRect( &rect,
                     GetSysColor(COLOR_BTNSHADOW),
                     GetSysColor(COLOR_BTNHIGHLIGHT) );
    }
else if( m_bMouse )
    {
    // define the colors for the top-left and bottom-right edges
    COLORREF colorTL, colorBR;

    if( m_bClick )
        {
        // mouse button is down, draw with "pushed-in" look
        colorTL = GetSysColor( COLOR_BTNSHADOW );
        colorBR = GetSysColor( COLOR_BTNHIGHLIGHT );
        }
    else
        {
        // mouse button is up, draw with "raised" look
        colorTL = GetSysColor( COLOR_BTNHIGHLIGHT );
        colorBR = GetSysColor( COLOR_BTNSHADOW );
        }

    GetBtnAreaRect( &rect );
    pDC->Draw3dRect( &rect, colorTL, colorBR );
    GetMenuAreaRect( &rect );
    pDC->Draw3dRect( &rect, colorTL, colorBR );
    }
}


///////////////////////////////////////////////////////////////////
// OnMouseMove - track mouse state and update appearance if reqd.
///////////////////////////////////////////////////////////////////
void CFlatButton::OnMouseMove( UINT nFlags, CPoint point )
{
if( !m_bMouse )
    {
    SetCapture();
    m_bMouse = TRUE;

    if( nFlags & MK_LBUTTON )
        {
        if( !m_bClick )
            // left button was held down outside client area
            return;
        }
    else
        // left button is up, reset "click" state
        m_bClick = FALSE;

    Invalidate();
    }
else
    {
    // release mouse capture if pointer is outside client area
    CRect rect;

    GetClientRect( &rect );
    if( !rect.PtInRect(point) )
        ReleaseCapture();
    }
}


///////////////////////////////////////////////////////////////////
// OnLButtonDown - set button state, display menu if available.
///////////////////////////////////////////////////////////////////
void CFlatButton::OnLButtonDown( UINT nFlags, CPoint point )
{
CButton::OnLButtonDown( nFlags, point );

CRect rect;

GetMenuAreaRect( &rect );
if( rect.PtInRect(point) )
    {
    // set menu state and update window appearance
    m_bMenu = TRUE;
    Invalidate();
    UpdateWindow();

    // display pop-up menu
    CMenu *pSub = m_pMenu->GetSubMenu( 0 );

    // must convert menu display location to screen coordinates
    GetClientRect( &rect );
    ClientToScreen( &rect );
    pSub->TrackPopupMenu( TPM_LEFTALIGN,
                          rect.left, rect.bottom,
                          AfxGetMainWnd() );

    // remove any more button down messages from queue
    MSG msg;

    PeekMessage( &msg, m_hWnd, WM_LBUTTONDOWN, WM_LBUTTONDOWN,
                 PM_REMOVE );

    // reset menu state and ensure appearance is updated
    m_bMenu = FALSE;
    Invalidate();
    }
else
    {
    // button click in non-menu area - set "click " state and
    // update appearance
    m_bClick = TRUE;
    Invalidate();
    UpdateWindow();
    }
}


///////////////////////////////////////////////////////////////////
// OnLButtonUp - reset button "clicked" state.
///////////////////////////////////////////////////////////////////
void CFlatButton::OnLButtonUp( UINT nFlags, CPoint point )
{
CButton::OnLButtonUp( nFlags, point );

m_bClick = FALSE;
Invalidate();
}


///////////////////////////////////////////////////////////////////
// OnLButtonDblClk - convert the doubleclick to a buttondown event.
///////////////////////////////////////////////////////////////////
void CFlatButton::OnLButtonDblClk( UINT nFlags, CPoint point )
{
OnLButtonDown( nFlags, point );
}


///////////////////////////////////////////////////////////////////
// OnCaptureChanged - reset internal mouse state if we lost capture
///////////////////////////////////////////////////////////////////
void CFlatButton::OnCaptureChanged( CWnd* pWnd )
{
CButton::OnCaptureChanged( pWnd );

if( pWnd != this )
    {
    m_bMouse = FALSE;
    Invalidate();
    }
}


///////////////////////////////////////////////////////////////////
// SetMenu - sets/resets the menu that the control will display.
///////////////////////////////////////////////////////////////////
BOOL CFlatButton::SetMenu( HMENU hMenu )
{
// remove current menu if any first
if( m_pMenu )
    {
    delete m_pMenu;
    m_pMenu = NULL;
    }

BOOL bRet;

if( hMenu == NULL )
    bRet = TRUE;
else
    {
    m_pMenu = new CMenu;
    bRet = m_pMenu->Attach( hMenu );
    }

// ensure that appearance is updated
Invalidate();
return bRet;
}


///////////////////////////////////////////////////////////////////
// SetBitmap - sets/resets the bitmap that the control will display
///////////////////////////////////////////////////////////////////
BOOL CFlatButton::SetBitmap( HBITMAP hBitmap )
{
// remove current bitmap if any first
if( m_pBitmap )
    {
    delete m_pBitmap;
    m_pBitmap = NULL;
    }

BOOL bRet;

if( hBitmap == NULL )
    bRet = TRUE;
else
    {
    m_pBitmap = new CBitmap;
    bRet = m_pBitmap->Attach( hBitmap );
    }

// ensure that appearance is updated
Invalidate();
return bRet;
}


///////////////////////////////////////////////////////////////////
// GetMenuAreaRect - returns the size of the control's "menu area".
///////////////////////////////////////////////////////////////////
inline void CFlatButton::GetMenuAreaRect( LPRECT pRect )
{
GetClientRect( pRect );
if( m_pMenu )
    // right-hand side of the control is the "menu area"
    pRect->left = pRect->right - m_nMenuAreaWidth;
else
    // no menu, so no menu area
    pRect->left = pRect->right;
}


///////////////////////////////////////////////////////////////////
// GetBtnAreaRect - returns the size of the control's "button area"
///////////////////////////////////////////////////////////////////
inline void CFlatButton::GetBtnAreaRect( LPRECT pRect )
{
GetClientRect( pRect );

if( m_pMenu )
    // if there is a menu then subtract the width of the menu area
    pRect->right -= m_nMenuAreaWidth;
}


// End of file //
